@@ -8,7 +8,14 @@ def df_transform(df):
 
 def add_same_indicator(df: pd.DataFrame) -> pd.DataFrame:
     """Adds values of same indicators due to several measure for a week"""
-    return df.groupby(['indicator', 'year_week', 'country'])[['value']].sum().reset_index()
+    df1 = df[df['indicator'].isin(['Daily hospital occupancy', 'Daily ICU occupancy'])]
+    df2 = df[df['indicator'].isin(['Weekly new hospital admissions per 100k', 'Weekly new ICU admissions per 100k'])]
+     
+    dff = pd.concat([
+      df1.groupby(['indicator', 'year_week', 'country'])[['value']].last(),
+      df2.groupby(['indicator', 'year_week', 'country'])[['value']].sum()
+    ]).reset_index()
+    return dff
 
 def rebuild_columns(df: pd.DataFrame) -> pd.DataFrame:
     """Creates 4 new columns for each inficator and transforms daily in weekly values"""
